/***************************************************************************
                          dchubprofile.cpp  -  description
                             -------------------
    begin                : Fre Sep 20 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpushbutton.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qinputdialog.h>

#include <dcconfig.h>

#include <dclib/core/cstringlist.h>

#include "dchubprofile.h"

/** */
DCHubProfile::DCHubProfile(QWidget *parent, const char *name ) : DCDialogHubProfile(parent,name)
{
	InitDocument();
}

/** */
DCHubProfile::~DCHubProfile()
{
}

/** */
void DCHubProfile::InitDocument()
{
	CStringList list;
	DCConfigHubProfile * pConfigHubProfile = 0;

	ComboBox_PROFILE->clear();

 	Config->GetHubProfileList( &list );

	while ( list.Next((CObject*&)pConfigHubProfile) )
	{
		if ( pConfigHubProfile->sName != "" )
			ComboBox_PROFILE->insertItem(pConfigHubProfile->sName.Data());
	}

	connect( PushButton_NEWPROFILE, SIGNAL(clicked()), this, SLOT(slotClickedNew()) );
	connect( PushButton_DELPROFILE, SIGNAL(clicked()), this, SLOT(slotClickedDel()) );
	connect( PushButton_OK, SIGNAL(clicked()), this, SLOT(slotClickedOK()) );
	connect( PushButton_CANCEL, SIGNAL(clicked()), this, SLOT(slotClickedCancel()) );

	connect( ComboBox_PROFILE, SIGNAL(activated( const QString & )), this, SLOT(slotProfileActiveted( const QString & )) );

	slotProfileActiveted("");
}

/** */
void DCHubProfile::SaveProfile( QString name )
{
	DCConfigHubProfile pConfigHubProfile;

	if ( name != "" )
	{
		pConfigHubProfile.sName        = name;
		pConfigHubProfile.sPassword    = LineEdit_PASSWORD->text().ascii();
		pConfigHubProfile.bAutoConnect = CheckBox_AUTOCONNECT->isChecked();

		Config->AddHubProfile( &pConfigHubProfile );
	}

	Config->SaveHubProfile();
}

/** */
void DCHubProfile::slotClickedNew()
{
	int i;
	DCConfigHubProfile ConfigHubProfile;
	bool ok = FALSE;

	QString text = QInputDialog::getText(
			tr( "Create Hub Profile" ),
			tr( "Please enter the profile name" ),
			QLineEdit::Normal, QString::null, &ok, this );

	if ( ok && !text.isEmpty() )
	{
		// check if entry exist
		for(i=0;i<ComboBox_PROFILE->count();i++)
		{
			if ( ComboBox_PROFILE->text(i) == text )
			{
				break;
			} 
		}

		// add new entry
		if ( i == ComboBox_PROFILE->count() )
		{
			ConfigHubProfile.sName = text.ascii();

			Config->AddHubProfile( &ConfigHubProfile );

			ComboBox_PROFILE->insertItem(text);

			for(i=0;i<ComboBox_PROFILE->count();i++)
			{
				if ( ComboBox_PROFILE->text(i) == text )
				{
					ComboBox_PROFILE->setCurrentItem(i);
					slotProfileActiveted("");
					break;
				} 
			}
		}
	}
}

/** */
void DCHubProfile::slotClickedDel()
{
	if ( !ComboBox_PROFILE->currentText().isEmpty() )
	{
		m_sOldName = "";

		Config->DelHubProfile( ComboBox_PROFILE->currentText().ascii() );
		ComboBox_PROFILE->removeItem( ComboBox_PROFILE->currentItem() );
		ComboBox_PROFILE->setCurrentItem(0);
		slotProfileActiveted("");
	}
}

/** */
void DCHubProfile::slotClickedCancel()
{
	reject();
}

/** */
void DCHubProfile::slotClickedOK()
{
	SaveProfile(ComboBox_PROFILE->currentText());

	accept();
}

/** */
void DCHubProfile::slotProfileActiveted( const QString & /*s*/ )
{
	DCConfigHubProfile pConfigHubProfile;

	if ( m_sOldName != "" )
	{
		SaveProfile(m_sOldName);
	}

	m_sOldName = ComboBox_PROFILE->currentText();

	// reset values
	LineEdit_PASSWORD->setText("");
	CheckBox_AUTOCONNECT->setChecked(FALSE);

	if ( m_sOldName != "" )
	{
		if ( Config->GetHubProfile( m_sOldName.ascii(), &pConfigHubProfile ) == TRUE )
		{
			LineEdit_PASSWORD->setText( pConfigHubProfile.sPassword.Data() );
			CheckBox_AUTOCONNECT->setChecked( pConfigHubProfile.bAutoConnect );
		}
	}
}
