/***************************************************************************
                          dcoptions.cpp  -  description
                             -------------------
    begin                : Sun Feb 24 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qdir.h>
#include <qdialog.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qcombobox.h>
#include <qlistbox.h>
#include <qslider.h>
#include <qtabwidget.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qspinbox.h>
#include <qstylefactory.h>
#include <qapplication.h>
#include <qlistview.h>
#include <qinputdialog.h>
#include <qfontdialog.h>
#include <qlayout.h>
#include <qsound.h>
#include <qregexp.h>
#include <qcolordialog.h>
#include <qtoolbutton.h>
#include <qimage.h>
#include <qcursor.h>
#include <qpopupmenu.h>

#include <cfloatspinbox.h>

#include <dclib/core/cbytearray.h>
#include <dclib/core/cstring.h>
#include <dclib/core/che3.h>
#include <dclib/core/csocket.h>
#include <dclib/core/cdir.h>
#include <dclib/cconfig.h>
#include <dclib/cfilemanager.h>

#include <dcbrowsefiletree.h>
#include <dcfiletool.h>
#include <dcfilebrowser.h>
#include <dcmenuhandler.h>

#include "dcoptions.h"

#include <dclib/dcos.h>
#include <dclib/cutils.h>
#include <math.h>

#include "res/configure.xpm"

/** */
DCOptions::DCOptions()
{
	// set default icon
	setIcon( QPixmap( configure_xpm ) );

	InitDocument();
}

/** */
DCOptions::~DCOptions()
{
}

/** */
void DCOptions::InitDocument()
{
	// set width mode
	ListView_OPTIONTREE->setColumnWidthMode( 0, QListView::Maximum );
	// hide the column header
	ListView_OPTIONTREE->header()->hide();
	// disable sort
	ListView_OPTIONTREE->setSorting(-1);

	// setup spin boxes
	SpinBox_MAXUPLOADRATE->setMinValue(0);
	SpinBox_MAXUPLOADRATE->setMaxValue(99999999);
	SpinBox_MAXDOWNLOADRATE->setMinValue(0);
	SpinBox_MAXDOWNLOADRATE->setMaxValue(99999999);

	// setup the option tree
	plviSecurity = new QListViewItem( ListView_OPTIONTREE, 0 );
	plviSecurity->setText( 0, tr( "Security" ) );

	plviOther = new QListViewItem( ListView_OPTIONTREE, 0 );
	plviOther->setText( 0, tr( "Other" ) );

	plviLog = new QListViewItem( ListView_OPTIONTREE, 0 );
	plviLog->setText( 0, tr( "Log" ) );

	plviSound = new QListViewItem( ListView_OPTIONTREE, 0 );
	plviSound->setText( 0, tr( "Sound" ) );

	plviGUI = new QListViewItem( ListView_OPTIONTREE, 0 );
	plviGUI->setText( 0, tr( "GUI" ) );
	plviGUIUser = new QListViewItem( plviGUI, 0 );
	plviGUIUser->setText( 0, tr( "User" ) );
	plviGUIChat = new QListViewItem( plviGUI, 0 );
	plviGUIChat->setText( 0, tr( "Chat" ) );
	plviGUITransfer = new QListViewItem( plviGUI, 0 );
	plviGUITransfer->setText( 0, tr( "Transfer" ) );
	plviGUIGeneral = new QListViewItem( plviGUI, 0 );
	plviGUIGeneral->setText( 0, tr( "General" ) );

	plviConnection = new QListViewItem( ListView_OPTIONTREE, 0 );
	plviConnection->setText( 0, tr( "Connection" ) );
	plviConnectionSettings = new QListViewItem( plviConnection, 0 );
	plviConnectionSettings->setText( 0, tr( "Settings" ) );
	plviConnectionMode = new QListViewItem( plviConnection, 0 );
	plviConnectionMode->setText( 0, tr( "Mode" ) );

	plviTransfer = new QListViewItem( ListView_OPTIONTREE, 0 );
	plviTransfer->setText( 0, tr( "Transfer" ) );
	plviTransferSettings = new QListViewItem( plviTransfer, 0 );
	plviTransferSettings->setText( 0, tr( "Settings" ) );
	plviTransferFolder = new QListViewItem( plviTransfer, 0 );
	plviTransferFolder->setText( 0, tr( "Folder" ) );

	plviDCGUI = new QListViewItem( ListView_OPTIONTREE, 0 );
	plviDCGUI->setText( 0, tr( "DCGUI" ) );
	plviDCGUIIdentify = new QListViewItem( plviDCGUI, 0 );
	plviDCGUIIdentify->setText( 0, tr( "Identify" ) );

	connect( ListView_OPTIONTREE, SIGNAL(clicked( QListViewItem * )), this, SLOT(slotListViewOptionClicked( QListViewItem * )) );

	connect( ToolButton_BROWSESOUNDFILECONNECT, SIGNAL(clicked()), this, SLOT(slotBrowseSoundFileConnect()) );
	connect( ToolButton_BROWSESOUNDFILEDISCONNECT, SIGNAL(clicked()), this, SLOT(slotBrowseSoundFileDisconnect()) );
	connect( ToolButton_BROWSESOUNDFILESEND, SIGNAL(clicked()), this, SLOT(slotBrowseSoundFileSend()) );
	connect( ToolButton_BROWSESOUNDFILERECEIVE, SIGNAL(clicked()), this, SLOT(slotBrowseSoundFileReceive()) );
	connect( ToolButton_BROWSESOUNDFILEFIRSTRECEIVE, SIGNAL(clicked()), this, SLOT(slotBrowseSoundFileFirstReceive()) );

	connect( ToolButton_PREVIEWSOUNDFILECONNECT, SIGNAL(clicked()), this, SLOT(slotPreviewSoundFileConnect()) );
	connect( ToolButton_PREVIEWSOUNDFILEDISCONNECT, SIGNAL(clicked()), this, SLOT(slotPreviewSoundFileDisconnect()) );
	connect( ToolButton_PREVIEWSOUNDFILESEND, SIGNAL(clicked()), this, SLOT(slotPreviewSoundFileSend()) );
	connect( ToolButton_PREVIEWSOUNDFILERECEIVE, SIGNAL(clicked()), this, SLOT(slotPreviewSoundFileReceive()) );
	connect( ToolButton_PREVIEWSOUNDFILEFIRSTRECEIVE, SIGNAL(clicked()), this, SLOT(slotPreviewSoundFileFirstReceive()) );

	connect( ToolButton_BROWSESOUNDPLAYER, SIGNAL(clicked()), this, SLOT(slotBrowseSoundPlayer()) );

	// Transfer->Folder
	connect( ToolButton_BROWSEDOWNLOADFOLDER, SIGNAL(clicked()), this, SLOT(slotBrowseDownloadFolder()) );
	connect( ToolButton_BROWSEDOWNLOADFINISHEDFOLDER, SIGNAL(clicked()), this, SLOT(slotBrowseDownloadFinishedFolder()) );

	connect( ToolButton_CREATESHARELIST, SIGNAL(clicked()), this, SLOT(slotCreateShareList()) );
	connect( ToolButton_BROWSEFILELIST, SIGNAL(clicked()), this, SLOT(slotBrowseShareList()) );
	connect( ToolButton_SHARELISTINFO, SIGNAL(clicked()), this, SLOT(slotShareListInfo()) );
	connect( ToolButton_ADDSHAREDFOLDER, SIGNAL(clicked()), this, SLOT(slotAddSharedFolder()) );
	connect( ToolButton_REMOVESHAREDFOLDER, SIGNAL(clicked()), this, SLOT(slotRemoveSharedFolder()) );
	connect( ToolButton_EDITSHAREDFOLDER, SIGNAL(clicked()), this, SLOT(slotEditSharedFolder()) );

	//
	connect( PushButton_SAVE, SIGNAL(clicked()), this, SLOT(slotSave()) );
	connect( PushButton_CANCEL, SIGNAL(clicked()), this, SLOT(reject()) );
	
	connect( PushButton_TESTWRONGIP, SIGNAL(clicked()), this, SLOT(slotTestWrongIp()) );
	connect( PushButton_GETINTERFACE, SIGNAL(clicked()), this, SLOT(slotGetInterface()) );
	connect( PushButton_TESTINTERFACE, SIGNAL(clicked()), this, SLOT(slotTestInterface()) );

	// GUI->General
	connect( ToolButton_BROWSELANGUAGEFILE, SIGNAL(clicked()), this, SLOT(slotBrowseLanguageFile()) );
	connect( ToolButton_SELECTLANGUAGE, SIGNAL(clicked()), this, SLOT(slotSelectLanguage()) );
	connect( PushButton_SETTHEME, SIGNAL(clicked()), this, SLOT(slotSetTheme()) );
	connect( ToolButton_BROWSEAPPFONT, SIGNAL(clicked()), this, SLOT(slotBrowseAppFont()) );
	connect( ToolButton_BROWSEDATAFOLDER, SIGNAL(clicked()), this, SLOT(slotBrowseDataFolder()) );

	// GUI->Chat
	connect( ToolButton_BROWSEBROWSER, SIGNAL(clicked()), this, SLOT(slotBrowseBrowser()) );
	connect( ListBox_CHATCOLOR, SIGNAL(doubleClicked(QListBoxItem *)), this, SLOT(slotDoubleClickedChatColor(QListBoxItem *)) );

	// GUI->Client
	connect( CheckBox_AUTOAWAYMODE, SIGNAL(clicked()), this, SLOT(slotAutoAwayMode()) );

	// LOG->Chat
	connect( CheckBox_LOGCHAT_ENABLELOGGING, SIGNAL(clicked()), this, SLOT(slotLogChatEnableLogging()) );
	connect( CheckBox_LOGCHAT_ENABLENICKNAMEFILTER, SIGNAL(clicked()), this, SLOT(slotLogChatEnableNickNameFilter()) );
	connect( ToolButton_LOGCHAT_NICKNAMEFILTER_ADD, SIGNAL(clicked()), this, SLOT(slotLogChatAddNickNameFilter()) );
	connect( ToolButton_LOGCHAT_NICKNAMEFILTER_DEL, SIGNAL(clicked()), this, SLOT(slotLogChatDelNickNameFilter()) );

	//
	connect( ToolButton_BROWSETRANSFERCERT, SIGNAL(clicked()), this, SLOT(slotBrowseTransferCert()) );
	connect( ToolButton_BROWSETRANSFERKEY, SIGNAL(clicked()), this, SLOT(slotBrowseTransferKey()) );

	connect( RadioButton_ACTIVEMODE, SIGNAL(toggled(bool)), this, SLOT(slotModeActiveChange(bool)) );
	connect( RadioButton_PASSIVEMODE, SIGNAL(toggled(bool)), this, SLOT(slotModePassiveChange(bool)) );
	connect( RadioButton_WRONGIP, SIGNAL(toggled(bool)), this, SLOT(slotWrongIPChange(bool)) );
	connect( RadioButton_INTERFACEIP, SIGNAL(toggled(bool)), this, SLOT(slotInterfaceIPChange(bool)) );

	connect( ToolButton_ADDHUBLISTURL, SIGNAL(clicked()), this, SLOT(slotAddHubListUrl()) );
	connect( ToolButton_REMOVEHUBLISTURL, SIGNAL(clicked()), this, SLOT(slotRemoveHubListUrl()) );

	connect( CheckBox_SOUNDDISABLED, SIGNAL(clicked()), this, SLOT(slotDisableSound()) );
	
	connect( CheckBox_EXTERNALPLAYER, SIGNAL(clicked()), this, SLOT(slotExternalPlayer()) );
	connect( CheckBox_SOUNDPLAYCONNECT, SIGNAL(clicked()), this, SLOT(slotSoundPlayConnect()) );
	connect( CheckBox_SOUNDPLAYDISCONNECT, SIGNAL(clicked()), this, SLOT(slotSoundPlayDisconnect()) );
	connect( CheckBox_SOUNDPLAYSEND, SIGNAL(clicked()), this, SLOT(slotSoundPlaySend()) );
	connect( CheckBox_SOUNDPLAYRECEIVE, SIGNAL(clicked()), this, SLOT(slotSoundPlayRecieve()) );
	connect( CheckBox_SOUNDPLAYFIRSTRECEIVE, SIGNAL(clicked()), this, SLOT(slotSoundPlayFirstReceive()) );

	connect( ToolButton_BROWSELOGFILE, SIGNAL(clicked()), this, SLOT(slotBrowseLogFile()) );

	// style options
	ComboBox_THEME->clear();

	if ( Config->GetThemeSupport() == TRUE )
	{
		QStringList list = QStyleFactory::keys();

		if ( list.count() > 0 )
		{
			list.sort();
			ComboBox_THEME->insertStringList(list);
			list.clear();
		}
	}
	else
	{
		// disable the theme support
		ComboBox_THEME->setEnabled(FALSE);
		PushButton_SETTHEME->setEnabled(FALSE);
	}
}

/** */
void DCOptions::slotSave()
{
	int tcpport, udpport;
	bool ok;

	// check if mode change
	if ( ((RadioButton_ACTIVEMODE->isOn() == TRUE) && (Config->GetMode() == ecmPASSIVE)) ||
	     ((RadioButton_ACTIVEMODE->isOn() == FALSE) && (Config->GetMode() == ecmACTIVE)) )
	{
		QMessageBox::warning( 0, "DCGUI-QT",
			QString(tr("You change the client mode ! Please restart DCGUI !")));
	}

	if ( LineEdit_DOWNLOADFOLDER->text() == "" )
	{
		QMessageBox::critical( 0, "DCGUI-QT",
		QString(tr("You must set a Download Folder !")));
		return;
	}

	if ( LineEdit_DOWNLOADFINISHEDFOLDER->text() == LineEdit_DOWNLOADFOLDER->text() )
	{
		QMessageBox::critical( 0, "DCGUI-QT",
		QString(tr("Download Finished Folder and Download Folder must be different !")));
		return;
	}

	// check active mode settings
	if ( RadioButton_ACTIVEMODE->isOn() == TRUE )
	{
		// check fake host
		if ( (RadioButton_WRONGIP->isOn() == TRUE) &&
	     	     (LineEdit_FAKEHOST->text() == "") )

		{
			QMessageBox::critical( 0, "DCGUI-QT",
			QString(tr("You must enter a valid 'Fake Host' !")));
			return;
		}

		// check interface
		if ( (RadioButton_INTERFACEIP->isOn() == TRUE) &&
	     	     (LineEdit_INTERFACE->text() == "") )

		{
			QMessageBox::critical( 0, "DCGUI-QT",
			QString(tr("You must enter a valid interface !")));
			return;
		}

		// check ports
		tcpport = LineEdit_TCPLISTENPORT->text().toInt(&ok);

		if ( (ok == FALSE) || (tcpport == 0) )
		{
			QMessageBox::critical( 0, "DCGUI-QT",
			QString(tr("You must enter a valid tcp listen port !")));
			return;
		}

		udpport = LineEdit_UDPLISTENPORT->text().toInt(&ok);

		if ( (ok == FALSE) || (udpport == 0) )
		{
			QMessageBox::critical( 0, "DCGUI-QT",
			QString(tr("You must enter a valid udp listen port !")));
			return;
		}

#ifndef WIN32
		if ( (tcpport < 1024) || (udpport < 1024) )
		{
			QMessageBox::critical( 0, "DCGUI-QT",
			QString(tr("You listen on a port < 1024 ! You can run dcgui only as root in active mode !")));
		}
#endif
	}

	// no nick is given
	if ( LineEdit_NICK->text() == "" )
	{
		QMessageBox::critical( 0, "DCGUI-QT",
		QString(tr("You must enter a valid nick !")));
		return;
	}

	// no search nick is given
	if ( (LineEdit_SEARCHNICK->text() == "") || (LineEdit_SEARCHNICK->text() == "nick_search") )
	{
		QMessageBox::critical( 0, "DCGUI-QT",
		QString(tr("You must enter a valid search nick !")));
		return;
	}

	if (LineEdit_APPFONT->text() != "")
	{
		QFont font;
		if (font.fromString(LineEdit_APPFONT->text()) == TRUE)
		{
			QApplication::setFont(font, TRUE);
		}
	}

	accept();
}

/** */
void DCOptions::slotAutoAwayMode()
{
	Frame_AUTOAWAYTIME->setEnabled(CheckBox_AUTOAWAYMODE->isChecked());
}

/** */
void DCOptions::slotDoubleClickedChatColor( QListBoxItem * item )
{
	QPixmap p(10,10);
	QColor color(item->pixmap()->convertToImage().pixel(0,0));
	color = QColorDialog::getColor(color);
	p.fill(color);
	ListBox_CHATCOLOR->changeItem( p, item->text(), ListBox_CHATCOLOR->currentItem() );
}

/** */
void DCOptions::slotLogChatEnableLogging()
{
	bool b;

	b = CheckBox_LOGCHAT_ENABLELOGGING->isChecked();

	CheckBox_LOGCHAT_APPENDDATE->setEnabled(b);
	CheckBox_LOGCHAT_APPENDHUBNAME->setEnabled(b);
	CheckBox_LOGCHAT_APPENDHUBHOST->setEnabled(b);
	CheckBox_LOGCHAT_DISABLEPUBLICCHAT->setEnabled(b);
	CheckBox_LOGCHAT_ENABLENICKNAMEFILTER->setEnabled(b);

	b = b & CheckBox_LOGCHAT_ENABLENICKNAMEFILTER->isChecked();

	GroupBox_LOGNICKNAMEFILTER->setEnabled(b);
}

/** */
void DCOptions::slotLogChatEnableNickNameFilter()
{
	bool b;

	b = CheckBox_LOGCHAT_ENABLENICKNAMEFILTER->isChecked();

	GroupBox_LOGNICKNAMEFILTER->setEnabled(b);
}

/** */
void DCOptions::slotLogChatAddNickNameFilter()
{
	bool ok = FALSE;

	QString text = QInputDialog::getText(
		tr("Add new nick name filter"),
		tr("Please enter a nick" ),
		QLineEdit::Normal, "", &ok, this );

	if ( ok && !text.isEmpty() )
	{
		if ( ListBox_LOGCHAT_NICKNAMEFILTER_LIST->findItem(text) == NULL )
		{
			ListBox_LOGCHAT_NICKNAMEFILTER_LIST->insertItem(text);
		}
	}
}

/** */
void DCOptions::slotLogChatDelNickNameFilter()
{
	int i;

	i = ListBox_LOGCHAT_NICKNAMEFILTER_LIST->currentItem();

	if ( i >= 0 )
	{
		ListBox_LOGCHAT_NICKNAMEFILTER_LIST->removeItem(i);
	}
}

/** */
void DCOptions::slotWrongIPChange( bool b )
{
	GroupBox_WRONGIP->setEnabled(b);
	GroupBox_INTERFACEIP->setEnabled(!b);
}

/** */
void DCOptions::slotInterfaceIPChange( bool b )
{
	GroupBox_WRONGIP->setEnabled(!b);
	GroupBox_INTERFACEIP->setEnabled(b);
}

/** */
void DCOptions::slotModeActiveChange(bool b)
{
	ButtonGroup_ACTIVEMODESETTINGS->setEnabled(b);
	ButtonGroup_PASSIVEMODESETTINGS->setEnabled(!b);
}

/** */
void DCOptions::slotModePassiveChange(bool b)
{
	ButtonGroup_ACTIVEMODESETTINGS->setEnabled(!b);
	ButtonGroup_PASSIVEMODESETTINGS->setEnabled(b);
}

/** */
void DCOptions::slotTestWrongIp()
{
	QString sip="";
	CSocket Socket;
	CString s;

	s = LineEdit_FAKEHOST->text().ascii();
	s = s.Replace(' ',"");
	LineEdit_FAKEHOST->setText(s.Data());

	if ( s != "" )
	{
		sip = CSocket::GetHostByName(s).Data();
	}

	if ( sip != "" )
	{
		QMessageBox::information( 0, "DCGUI-QT", QString(tr("Host IP : "))+sip);
	}
	else
	{
		QMessageBox::critical( 0, "DCGUI-QT", QString(tr("Can't get Host IP !")));
	}
}

/** */
void DCOptions::slotGetInterface()
{
	CSocket Socket;
	CList<CString> interfacelist;
	CString * s;
	QStringList lst;

	if ( Socket.GetInterfaceList(&interfacelist) > 0 )
	{
		s = 0;

		while( (s=interfacelist.Next(s)) != 0 )
		{
			lst << s->Data();
		}
  
		bool ok = FALSE;

		QString res = QInputDialog::getItem(
				tr( "Interface List" ),
				tr( "Please select an interface" ), lst, 1, TRUE, &ok, this );

		if ( ok )
		{
			LineEdit_INTERFACE->setText(res);
		}
	}
	else
	{
		QMessageBox::information( 0, "DCGUI-QT", QString(tr("No interfaces found !")));
	}
}

/** */
void DCOptions::slotTestInterface()
{
	QString sip;
	CSocket Socket;

	sip = Socket.GetInterfaceIP(LineEdit_INTERFACE->text().ascii()).Data();

	if ( sip != "" )
	{
		QMessageBox::information( 0, "DCGUI-QT", QString(tr("Interface IP : "))+sip);
	}
	else
	{
		QMessageBox::critical( 0, "DCGUI-QT", QString(tr("Can't get interface IP !")));
	}
}

/** */
void DCOptions::slotDisableSound()
{
	bool b;

	b = !CheckBox_SOUNDDISABLED->isChecked();

	CheckBox_SOUNDDISABLEDAWAY->setEnabled(b);
	CheckBox_SOUNDPLAYCONNECT->setEnabled(b);
	CheckBox_SOUNDPLAYDISCONNECT->setEnabled(b);
	CheckBox_SOUNDPLAYSEND->setEnabled(b);
	CheckBox_SOUNDPLAYRECEIVE->setEnabled(b);
	CheckBox_SOUNDPLAYFIRSTRECEIVE->setEnabled(b);
	CheckBox_EXTERNALPLAYER->setEnabled(b);

	if (!b) {
		LineEdit_EXTERNALPLAYER->setEnabled(b);
		ToolButton_BROWSESOUNDPLAYER->setEnabled(b);
		LineEdit_SOUNDFILECONNECT->setEnabled(b);
		ToolButton_BROWSESOUNDFILECONNECT->setEnabled(b);
		ToolButton_PREVIEWSOUNDFILECONNECT->setEnabled(b);
		LineEdit_SOUNDFILEDISCONNECT->setEnabled(b);
		ToolButton_BROWSESOUNDFILEDISCONNECT->setEnabled(b);
		ToolButton_PREVIEWSOUNDFILEDISCONNECT->setEnabled(b);
		LineEdit_SOUNDFILESEND->setEnabled(b);
		ToolButton_BROWSESOUNDFILESEND->setEnabled(b);
		ToolButton_PREVIEWSOUNDFILESEND->setEnabled(b);
		LineEdit_SOUNDFILERECEIVE->setEnabled(b);
		ToolButton_BROWSESOUNDFILERECEIVE->setEnabled(b);
		ToolButton_PREVIEWSOUNDFILERECEIVE->setEnabled(b);
		LineEdit_SOUNDFILEFIRSTRECEIVE->setEnabled(b);
		ToolButton_BROWSESOUNDFILEFIRSTRECEIVE->setEnabled(b);
		ToolButton_PREVIEWSOUNDFILEFIRSTRECEIVE->setEnabled(b);
	} else {
		slotExternalPlayer();
		slotSoundPlayConnect();
		slotSoundPlayConnect();
		slotSoundPlayDisconnect();
		slotSoundPlaySend();
		slotSoundPlayRecieve();
		slotSoundPlayFirstReceive();
	}
}

/** */
void DCOptions::slotExternalPlayer()
{
	bool b;

	b = CheckBox_EXTERNALPLAYER->isChecked();
	
	LineEdit_EXTERNALPLAYER->setEnabled(b);
	ToolButton_BROWSESOUNDPLAYER->setEnabled(b);
}

/** */
void DCOptions::slotSoundPlayConnect()
{
	bool b;

	b = CheckBox_SOUNDPLAYCONNECT->isChecked();
	
	LineEdit_SOUNDFILECONNECT->setEnabled(b);
	ToolButton_BROWSESOUNDFILECONNECT->setEnabled(b);
	ToolButton_PREVIEWSOUNDFILECONNECT->setEnabled(b);
}

/** */
void DCOptions::slotSoundPlayDisconnect()
{
	bool b;

	b = CheckBox_SOUNDPLAYDISCONNECT->isChecked();
	
	LineEdit_SOUNDFILEDISCONNECT->setEnabled(b);
	ToolButton_BROWSESOUNDFILEDISCONNECT->setEnabled(b);
	ToolButton_PREVIEWSOUNDFILEDISCONNECT->setEnabled(b);
}

/** */
void DCOptions::slotSoundPlaySend()
{
	bool b;

	b = CheckBox_SOUNDPLAYSEND->isChecked();
	
	LineEdit_SOUNDFILESEND->setEnabled(b);
	ToolButton_BROWSESOUNDFILESEND->setEnabled(b);
	ToolButton_PREVIEWSOUNDFILESEND->setEnabled(b);
}

/** */
void DCOptions::slotSoundPlayRecieve()
{
	bool b;

	b = CheckBox_SOUNDPLAYRECEIVE->isChecked();
	
	LineEdit_SOUNDFILERECEIVE->setEnabled(b);
	ToolButton_BROWSESOUNDFILERECEIVE->setEnabled(b);
	ToolButton_PREVIEWSOUNDFILERECEIVE->setEnabled(b);
}

/** */
void DCOptions::slotSoundPlayFirstReceive()
{
	bool b;

	b = CheckBox_SOUNDPLAYFIRSTRECEIVE->isChecked();
	
	LineEdit_SOUNDFILEFIRSTRECEIVE->setEnabled(b);
	ToolButton_BROWSESOUNDFILEFIRSTRECEIVE->setEnabled(b);
	ToolButton_PREVIEWSOUNDFILEFIRSTRECEIVE->setEnabled(b);
}

/** */
void DCOptions::slotBrowseSoundPlayer()
{
	QString file;

	file = QFileDialog::getOpenFileName( "", "", this, "blf", tr("Select sound player") );

	if( file != "" )
	{
		LineEdit_EXTERNALPLAYER->setText(file);
	}
}

/** */
void DCOptions::slotBrowseTransferCert()
{
	QString file;

	file = QFileDialog::getOpenFileName( "", "", this, "blf", tr("Select transfer cert") );

	if( file != "" )
	{
		LineEdit_TRANSFERCERT->setText(file);
	}
}

/** */
void DCOptions::slotBrowseTransferKey()
{
	QString file;

	file = QFileDialog::getOpenFileName( "", "", this, "blf", tr("Select transfer key") );

	if( file != "" )
	{
		LineEdit_TRANSFERKEY->setText(file);
	}
}

/** */
void DCOptions::slotBrowseBrowser()
{
	QString file;

	file = QFileDialog::getOpenFileName( "", "", this, "blf", tr("Select a browser") );

	if( file != "" )
	{
		LineEdit_BROWSER->setText(file);
	}
}

/** */
void DCOptions::BrowseSoundFile( eUserSound usersound )
{
	QString file;
	QString filter;

	filter = tr("au")        + " (*.au);;"  +
		 tr("wav")       + " (*.wav);;" +
		 tr("All Files") + " (*)";

	file = QFileDialog::getOpenFileName( "", filter, this, "blf", tr("Select sound file") );

	if( file != "" )
	{
		switch(usersound)
		{
			case eusCONNECT:
				LineEdit_SOUNDFILECONNECT->setText(file);
				break;
			case eusDISCONNECT:
				LineEdit_SOUNDFILEDISCONNECT->setText(file);
				break;
			case eusSEND:
				LineEdit_SOUNDFILESEND->setText(file);
				break;
			case eusRECEIVE:
				LineEdit_SOUNDFILERECEIVE->setText(file);
				break;
			case eusFIRSTRECEIVE:
				LineEdit_SOUNDFILEFIRSTRECEIVE->setText(file);
				break;
			default:
				break;
		}
	}
}

/** */
void DCOptions::PreviewSoundFile( eUserSound usersound )
{
	QString file="";

	switch(usersound)
	{
		case eusCONNECT:
			file = LineEdit_SOUNDFILECONNECT->text();
			break;
		case eusDISCONNECT:
			file = LineEdit_SOUNDFILEDISCONNECT->text();
			break;
		case eusSEND:
			file = LineEdit_SOUNDFILESEND->text();
			break;
		case eusRECEIVE:
			file = LineEdit_SOUNDFILERECEIVE->text();
			break;
		case eusFIRSTRECEIVE:
			file = LineEdit_SOUNDFILEFIRSTRECEIVE->text();
			break;
		default:
			break;
	}

	Config->PlaySound( file.ascii() );
}

/** */
void DCOptions::slotBrowseDownloadFolder()
{
	QString dir;

	dir = QFileDialog::getExistingDirectory ( "", this, "bdf", tr("Select download folder"), TRUE );

	if(dir!="")
	{
		LineEdit_DOWNLOADFOLDER->setText(dir);
	}
}

/** */
void DCOptions::slotBrowseDownloadFinishedFolder()
{
	QString dir;

	dir = QFileDialog::getExistingDirectory ( "", this, "bdf", tr("Select download finished folder"), TRUE );

	if(dir!="")
	{
		LineEdit_DOWNLOADFINISHEDFOLDER->setText(dir);
	}
}

/** */
void DCOptions::slotBrowseDataFolder()
{
	QString dir;

	dir = QFileDialog::getExistingDirectory ( "", this, "bdf", tr("Select data folder"), TRUE );

	if( dir != "" )
	{
		LineEdit_DATAFOLDER->setText(dir);
	}
}

/** */
void DCOptions::slotBrowseShareList()
{
	bool err = FALSE;
	DCFileBrowser *bft;
	CHE3 *he3=0;
	CByteArray ba;
	CString * s=0;

	if ( CFileManager::Instance()->GetShareBuffer( esbtHE3, &ba ) == 0 )
	{
		he3 = new CHE3();
		s = he3->decode_he3_data(&ba);
		delete he3;

		if ( s != 0 )
		{
			QDialog *d = new QDialog(this);
			QGridLayout *Layout = new QGridLayout( d, 1, 1, 4, 4, "Layout"); 
//			bft = new DCBrowseFileTree( d, "browsefiletree", 0, TRUE );
			bft = new DCFileBrowser( d, "browsefiletree" );
			Layout->addWidget( bft, 0, 0 );

			// set transfer
			d->setCaption( tr("My") + " - Filebrowser" );
			bft->InitTree("","","",s->Data());
			bft->layout()->setSpacing(0);
			bft->layout()->setMargin(0);
			d->exec();
			delete d;

			delete s;
		}
		else
		{
			err = TRUE;
		}
	}
	else
	{
		err = TRUE;
	}

	if ( err )
	{
		QMessageBox::information( this, tr("Browse Share List"), tr("Share list not available") );
	}
}

/** */
void DCOptions::slotBrowseLogFile()
{
	QString logfile;

	logfile = QFileDialog::getSaveFileName ( "", "", this, 0, tr("Select log file") );

	if( !logfile.isEmpty() )
	{
		LineEdit_LOGFILENAME->setText(logfile);
	}
}

/** */
void DCOptions::slotAddSharedFolder()
{
	CDir di;
	QString dir;
	CString s;
	int i;

	dir = QFileDialog::getExistingDirectory ( "", this, "bdf", tr("Select share folder"), TRUE );

	if( dir != "" )
	{
		if ( ListView_SHAREDFOLDERS->findItem( dir, 0 ) == 0 )
		{
			di.SetPath(dir.ascii());
			s = di.Path();
			i = 1;

			while( ListView_SHAREDFOLDERS->findItem( di.DirName().Data(), 1 ) != 0 )
			{
				di.SetPath( s + CString().setNum(i) );
				i++;
			}

			new QListViewItem( ListView_SHAREDFOLDERS, dir, di.DirName().Data() );
		}
	}
}

/** */
void DCOptions::slotRemoveSharedFolder()
{
	QListViewItem * item = ListView_SHAREDFOLDERS->currentItem();

	if ( item )
	{
		ListView_SHAREDFOLDERS->takeItem(item);
		delete item;
	}
}

/** */
void DCOptions::slotEditSharedFolder()
{
	bool ok = FALSE;
	QListViewItem * item = ListView_SHAREDFOLDERS->currentItem();

	if ( item )
	{
		QString text = QInputDialog::getText(
			tr("Edit Alias"),
			tr("Please enter a new alias" ),
			QLineEdit::Normal, item->text(1), &ok, this );

		if ( ok && !text.isEmpty() )
		{
			if ( ListView_SHAREDFOLDERS->findItem( text, 1 ) == 0 )
			{
				text = text.replace( QRegExp("\\"),"");
				text = text.replace( QRegExp("/"),"");
				item->setText(1,text);
			}
		}
	}
}

/** */
void DCOptions::SetValues()
{
	int i;
	CList<DCConfigShareFolder> list;
	DCConfigShareFolder * csf;

	// identify
	LineEdit_NICK->setText(Config->GetNick().Data());
	LineEdit_SEARCHNICK->setText(Config->GetSearchNick().Data());
	LineEdit_EMAIL->setText(Config->GetEMail(TRUE).Data());
	CheckBox_ANTISPAM->setChecked(Config->GetAntiSpam());
	LineEdit_DESCRIPTION->setText(Config->GetDescription(TRUE).Data());
	CheckBox_DESCRIPTION_TAG->setChecked(Config->GetDescriptionTag());
	LineEdit_AWAYPREFIX->setText(Config->GetAwayPrefix().Data());

	for(i=0;i<ComboBox_SPEED->count();i++)
	{
		if ( ComboBox_SPEED->text(i) == Config->GetSpeed() )
		{
			ComboBox_SPEED->setCurrentItem(i);
			break;
		}
	}

	TextEdit_AWAYMESSAGE->setText(QString(Config->GetAwayMessage().Data()).replace( QRegExp("\\r"), "" ));

	// transfer->folder
	LineEdit_DOWNLOADFOLDER->setText(Config->GetDownloadFolder().Data());
	LineEdit_DOWNLOADFINISHEDFOLDER->setText(Config->GetDownloadFinishedFolder().Data());

	// save shared folders
	ListView_SHAREDFOLDERS->clear();

	Config->GetSharedFolders(&list);

	csf = 0;
	while( (csf=list.Next(csf)) != 0 )	
	{
		new QListViewItem( ListView_SHAREDFOLDERS, csf->m_sPath.Data(), csf->m_sAlias.Data() );
	}

	CheckBox_AUTORECREATESHARELIST->setChecked(Config->GetAutoRecreateShareList());

	// transfer->settings
	SpinBox_SAVEDOWNLOADQUEUETIME->setValue( Config->GetDownloadQueueTime() );
	SpinBox_USERUPLOADSLOTS->setValue(Config->GetUserUploadSlots());
	CheckBox_DYNAMICUPLOADRATE->setChecked(Config->GetDynamicUploadRate());
	SpinBox_MAXUPLOADRATE->setValue( floor(((10.0*Config->GetMaxUploadRate())/1024)+0.5) );
	SpinBox_MAXDOWNLOADRATE->setValue( floor(((10.0*Config->GetMaxDownloadRate())/1024)+0.5) );
	SpinBox_MAXSIMULTANEOUSUPLOADS->setValue( Config->GetMaxUpload() );
	SpinBox_TRANSFERRESENDTIMEOUT->setValue(Config->GetTransferResendTimeout());
	SpinBox_TRANSFERRESPONSETIMEOUT->setValue(Config->GetTransferResponseTimeout());

	switch ( Config->GetHubOfflineTransferClose() )
	{
		case ectNONE:
			RadioButton_NONE->setChecked(TRUE);
			break;
		case ectUPLD:
			RadioButton_UPLOAD->setChecked(TRUE);
			break;
		case ectDLD:
			RadioButton_DOWNLOAD->setChecked(TRUE);
			break;
		case ectBOTH:
			RadioButton_BOTH->setChecked(TRUE);
			break;
	}

	CheckBox_TRANSFERAUTOSEARCH->setChecked(Config->GetTransferAutoSearch());
	SpinBox_RECREATESHARELISTTIME->setValue(Config->GetRecreateShareListTime());

	// connection->mode
	if ( Config->GetMode() == ecmPASSIVE )
	{
		RadioButton_PASSIVEMODE->setChecked(TRUE);
		slotModeActiveChange(FALSE);
	}
	else
	{
		RadioButton_ACTIVEMODE->setChecked(TRUE);
		slotModeActiveChange(TRUE);
	}

	if ( Config->GetFakeHost() == 1 )
	{
		RadioButton_WRONGIP->setChecked(TRUE);
		slotWrongIPChange(TRUE);
	}
	else
	{
		RadioButton_INTERFACEIP->setChecked(TRUE);
		slotWrongIPChange(FALSE);
	}

	LineEdit_FAKEHOST->setText(Config->GetHost().Data());
	LineEdit_INTERFACE->setText(Config->GetInterface().Data());
	LineEdit_TCPLISTENPORT->setText( QString().setNum(Config->GetTCPListenPort()) );
	LineEdit_UDPLISTENPORT->setText( QString().setNum(Config->GetUDPListenPort()) );
	CheckBox_SENDMESSAGEONACTIVEMODEREQUEST->setChecked( Config->GetSendMessageOnActiveModeRequest() );

	// connection->settings
	SpinBox_RECONNECTCOUNT->setValue(Config->GetReconnectCount());
	SpinBox_RECONNECTTIMEOUT->setValue(Config->GetReconnectTimeout());
	CheckBox_FORCEMOVEENABLED->setChecked(Config->GetForceMoveEnabled());
	CheckBox_CHECKPRIVATEADDRESSSPACE->setChecked(Config->GetCheckPrivateAddressSpace());

	// gui->general
	LineEdit_LANGUAGEFILE->setText( Config->GetLanguageFile().Data() );

	for(i=0;i<ComboBox_THEME->count();i++)
	{
		if ( Config->GetTheme() == CString(ComboBox_THEME->text(i).ascii()) )
		{
			ComboBox_THEME->setCurrentItem(i);
			break;
		}
	}

	switch(Config->GetUnit())
	{
		case euBYTE:
			RadioButton_UNITBYTE->setChecked(TRUE);
			break;
		case euKBYTE:
			RadioButton_UNITKBYTE->setChecked(TRUE);
			break;
		case euMBYTE:
			RadioButton_UNITMBYTE->setChecked(TRUE);
			break;
		case euGBYTE:
			RadioButton_UNITGBYTE->setChecked(TRUE);
			break;
		default:
			RadioButton_UNITAUTO->setChecked(TRUE);
			break;
	}

	LineEdit_APPFONT->setText( Config->GetAppFont().Data() );
	LineEdit_DATAFOLDER->setText(Config->GetDatadir().Data());
	CheckBox_SHOWSTATUSMESSAGE->setChecked( Config->GetShowStatusMessage() );
	CheckBox_QUERYONEXIT->setChecked( Config->GetQueryOnExit() );

	// gui->transfer
	CheckBox_TV_PERCENTCHUNK->setChecked(Config->GetTransferViewOptions(etvoCHUNKPERCENT));
	CheckBox_TV_PERCENTFILE->setChecked(Config->GetTransferViewOptions(etvoFILEPERCENT));
	CheckBox_TV_CHUNKSIZE->setChecked(Config->GetTransferViewOptions(etvoCHUNKSIZE));
	CheckBox_TV_FILESIZE->setChecked(Config->GetTransferViewOptions(etvoFILESIZE));
	CheckBox_TV_DOWNLOADRATESINGLE->setChecked(Config->GetTransferViewOptions(etvoDOWNLOADRATESINGLE));
	CheckBox_TV_ELAPSEDTIMESINGLE->setChecked(Config->GetTransferViewOptions(etvoELAPSEDTIMESINGLE));
	CheckBox_TV_DOWNLOADRATEMULTI->setChecked(Config->GetTransferViewOptions(etvoDOWNLOADRATEMULTI));
	CheckBox_TV_ELAPSEDTIMEMULTI->setChecked(Config->GetTransferViewOptions(etvoELAPSEDTIMEMULTI));

	// gui->chat
	CheckBox_OPENPRIVATECHATWINDOW->setChecked(Config->GetOpenPrivateChatWindow());
	CheckBox_SHOWCHATINTAB->setChecked(Config->GetShowChatInTab());
	CheckBox_SENDHIDEPRIVATECHATTOPUBLICCHAT->setChecked( Config->GetSendHidePrivateChatToPublicChat());
	CheckBox_EMOTICONS->setChecked(Config->GetEnableEmoticons());
	CheckBox_SHOWCHATSENDBUTTON->setChecked(Config->GetShowChatSendButton());
	CheckBox_CHATSHOWJOINSANDPARTS->setChecked(Config->GetChatShowJoinsAndParts());
	CheckBox_CHATRECVOFFLINEMESSAGES->setChecked(Config->GetChatRecvOfflineMessages());
	CheckBox_CHATSENDOFFLINEMESSAGES->setChecked(Config->GetChatSendOfflineMessages());
	SpinBox_CHATMAXPARAGRAPH->setValue(Config->GetChatMaxParagraph());
	SpinBox_CHATMESSAGEMAXLINES->setValue( Config->GetChatMessageMaxLines() );

	for(i=0;i<ComboBox_SENDCHAT->count();i++)
	{
		if ( Config->GetSendChat() == CString(ComboBox_SENDCHAT->text(i).ascii()) )
		{
			ComboBox_SENDCHAT->setCurrentItem(i);
			break;
		}
	}

	LineEdit_BROWSER->setText(Config->GetBrowser().Data());

	QColor c;
	QPixmap p(10,10);

	c.setNamedColor(Config->GetChatColor(eccCHATTIMESTAMP).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Chat Timestamp"));
	c.setNamedColor(Config->GetChatColor(eccCHATLOCALNICK).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Chat Local Nick"));
	c.setNamedColor(Config->GetChatColor(eccCHATLOCALTEXT).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Chat Local Text"));
	c.setNamedColor(Config->GetChatColor(eccCHATREMOTENICK).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Chat Remote Nick"));
	c.setNamedColor(Config->GetChatColor(eccCHATREMOTETEXT).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Chat Remote Text"));
	c.setNamedColor(Config->GetChatColor(eccCHATSTATUSNICK).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Chat Status Nick"));
	c.setNamedColor(Config->GetChatColor(eccCHATSTATUSTEXT).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Chat Status Text"));
	c.setNamedColor(Config->GetChatColor(eccPUBLICCHATTIMESTAMP).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Public Chat Timestamp"));
	c.setNamedColor(Config->GetChatColor(eccPUBLICCHATLOCALNICK).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Public Chat Local Nick"));
	c.setNamedColor(Config->GetChatColor(eccPUBLICCHATLOCALTEXT).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Public Chat Local Text"));
	c.setNamedColor(Config->GetChatColor(eccPUBLICCHATREMOTENICK).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Public Chat Remote Nick"));
	c.setNamedColor(Config->GetChatColor(eccPUBLICCHATREMOTETEXT).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Public Chat Remote Text"));
	c.setNamedColor(Config->GetChatColor(eccPUBLICCHATSTATUSNICK).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Public Chat Status Nick"));
	c.setNamedColor(Config->GetChatColor(eccPUBLICCHATSTATUSTEXT).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Public Chat Status Text"));
	c.setNamedColor(Config->GetChatColor(eccPUBLICPRIVATECHATNICK).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Public Private Chat Nick"));
	c.setNamedColor(Config->GetChatColor(eccPUBLICPRIVATECHATTEXT).Data());
	p.fill(c);
	ListBox_CHATCOLOR->insertItem(p,tr("Public Private Chat Text"));

	// gui->client
	CheckBox_CLIENTWINDOWSMINIMIZED->setChecked(Config->GetOpenClientWindowsMinimized());
	CheckBox_SHOWVERSIONTAG->setChecked(Config->GetShowVersionTag());
	CheckBox_USERLISTRIGHTALIGNMENT->setChecked(Config->GetUserListRightAlignment());
	CheckBox_AUTOAWAYMODE->setChecked(Config->GetAutoAwayMode());
	SpinBox_AUTOAWAYTIME->setValue(Config->GetAutoAwayTime());

	// sound
	CheckBox_SOUNDDISABLED->setChecked(Config->GetSoundDisabled());
	CheckBox_SOUNDDISABLEDAWAY->setChecked(Config->GetSoundDisabledAway());
	CheckBox_EXTERNALPLAYER->setChecked(Config->GetExternalSoundPlayer());
	LineEdit_EXTERNALPLAYER->setText( Config->GetExternalPlayer().Data() );
	LineEdit_SOUNDFILECONNECT->setText( Config->GetSoundFile(eusCONNECT).Data() );
	LineEdit_SOUNDFILEDISCONNECT->setText( Config->GetSoundFile(eusDISCONNECT).Data() );
	LineEdit_SOUNDFILESEND->setText( Config->GetSoundFile(eusSEND).Data() );
	LineEdit_SOUNDFILERECEIVE->setText( Config->GetSoundFile(eusRECEIVE).Data() );
	LineEdit_SOUNDFILEFIRSTRECEIVE->setText( Config->GetSoundFile(eusFIRSTRECEIVE).Data() );
	CheckBox_SOUNDPLAYCONNECT->setChecked(Config->GetSoundEnabled(eusCONNECT));
	CheckBox_SOUNDPLAYDISCONNECT->setChecked(Config->GetSoundEnabled(eusDISCONNECT));
	CheckBox_SOUNDPLAYSEND->setChecked(Config->GetSoundEnabled(eusSEND));
	CheckBox_SOUNDPLAYRECEIVE->setChecked(Config->GetSoundEnabled(eusRECEIVE));
	CheckBox_SOUNDPLAYFIRSTRECEIVE->setChecked(Config->GetSoundEnabled(eusFIRSTRECEIVE));

	// log
	CheckBox_LOGFILE->setChecked(Config->GetLogFile());
	LineEdit_LOGFILENAME->setText( Config->GetLogFileName().Data() );
	CheckBox_TIMESTAMPPRIVATECHAT->setChecked(Config->GetTimeStamp(etsPRIVATECHAT));
	CheckBox_TIMESTAMPHUBCHAT->setChecked(Config->GetTimeStamp(etsHUBCHAT));

	CheckBox_LOGCHAT_ENABLELOGGING->setChecked( Config->GetLogChatOption(elcoENABLELOGGING) );
	CheckBox_LOGCHAT_APPENDDATE->setChecked( Config->GetLogChatOption(elcoAPPENDDATE) );
	CheckBox_LOGCHAT_APPENDHUBNAME->setChecked( Config->GetLogChatOption(elcoAPPENDHUBNAME) );
	CheckBox_LOGCHAT_APPENDHUBHOST->setChecked( Config->GetLogChatOption(elcoAPPENDHUBHOST) );
	CheckBox_LOGCHAT_DISABLEPUBLICCHAT->setChecked( Config->GetLogChatOption(elcoDISABLEPUBLICCHAT) );
	CheckBox_LOGCHAT_ENABLENICKNAMEFILTER->setChecked( Config->GetLogChatOption(elcoENABLENICKNAMEFILTER) );
	CheckBox_LOGCHAT_NICKNAMEFILTER_ENABLELOGGING->setChecked( Config->GetLogChatOption(elcoNICKNAMEFILTER_ENABLELOGGING) );

	CList<CString> nnflist;
	CString * ps = 0;
	Config->GetLogChatNickNameFilter( &nnflist );

	ListBox_LOGCHAT_NICKNAMEFILTER_LIST->clear();

	while( (ps=nnflist.Next(ps)) != 0 )
	{
		ListBox_LOGCHAT_NICKNAMEFILTER_LIST->insertItem(ps->Data());
	}

	// other
	CheckBox_HUBLISTSTORELOCAL->setChecked(Config->GetHubListStoreLocal());
	SpinBox_RELOADHUBLISTTIME->setValue(Config->GetReloadHubListTime());

	DCConfigHubListUrl * hublisturl = 0;

	CList<DCConfigHubListUrl> hublisturllist;
	Config->GetHubListUrlList(&hublisturllist);

	while( (hublisturl=hublisturllist.Next(hublisturl)) != 0 )
	{
		QCheckListItem * cli = new QCheckListItem( ListView_HUBLISTURL, hublisturl->sUrl.Data(), QCheckListItem::CheckBox );
		cli->setOn(hublisturl->bEnabled);
	}

	// security
	SpinBox_FLOODCOUNT->setValue(Config->GetFloodCount());
	CheckBox_FLOODOPKICK->setChecked(Config->GetFloodOpKick());
	LineEdit_FLOODOPKICKMESSAGE->setText(QString(Config->GetFloodOpKickMessage().Data()).replace( QRegExp("\\r"), "" ));
	LineEdit_TRANSFERCERT->setText(Config->GetTransferCert().Data());
	LineEdit_TRANSFERKEY->setText(Config->GetTransferKey().Data());

	// init the sound tab
	slotDisableSound();
	// init log tab
	slotLogChatEnableLogging();
	// init away
	slotAutoAwayMode();
}

/** */
void DCOptions::GetValues()
{
	bool ok;

	Config->SetNick(LineEdit_NICK->text().ascii());
	Config->SetSearchNick(LineEdit_SEARCHNICK->text().ascii());
	Config->SetAwayMessage(TextEdit_AWAYMESSAGE->text().replace( QRegExp("\\n"), "\xd\xa" ).ascii());
	Config->SetDescription(LineEdit_DESCRIPTION->text().ascii());
	Config->SetAwayPrefix(LineEdit_AWAYPREFIX->text().ascii());
	Config->SetDescriptionTag(CheckBox_DESCRIPTION_TAG->isChecked());
	Config->SetEMail(LineEdit_EMAIL->text().ascii());
	Config->SetAntiSpam(CheckBox_ANTISPAM->isChecked());
	Config->SetCheckPrivateAddressSpace(CheckBox_CHECKPRIVATEADDRESSSPACE->isChecked());
	Config->SetShowChatSendButton(CheckBox_SHOWCHATSENDBUTTON->isChecked());
	Config->SetBrowser(LineEdit_BROWSER->text().ascii());
	Config->SetDynamicUploadRate(CheckBox_DYNAMICUPLOADRATE->isChecked());
	Config->SetShowChatInTab(CheckBox_SHOWCHATINTAB->isChecked());
	Config->SetChatMaxParagraph(SpinBox_CHATMAXPARAGRAPH->value());
	Config->SetTransferAutoSearch(CheckBox_TRANSFERAUTOSEARCH->isChecked());

	Config->SetSpeed(ComboBox_SPEED->currentText().ascii());
	Config->SetDownloadFolder(LineEdit_DOWNLOADFOLDER->text().ascii());
	Config->SetDownloadFinishedFolder(LineEdit_DOWNLOADFINISHEDFOLDER->text().ascii());
	Config->SetDatadir(LineEdit_DATAFOLDER->text().ascii());
	
	Config->SetAutoRecreateShareList(CheckBox_AUTORECREATESHARELIST->isChecked());
	Config->SetRecreateShareListTime(SpinBox_RECREATESHARELISTTIME->value());

	Config->SetFloodOpKickMessage(LineEdit_FLOODOPKICKMESSAGE->text().replace( QRegExp("\\n"), "\xd\xa" ).ascii());
	Config->SetFloodCount(SpinBox_FLOODCOUNT->value());
	Config->SetFloodOpKick(CheckBox_FLOODOPKICK->isChecked());

	Config->SetTransferCert(LineEdit_TRANSFERCERT->text().ascii());
	Config->SetTransferKey(LineEdit_TRANSFERKEY->text().ascii());

	SaveSharedFolders();

	Config->SetMaxUpload( SpinBox_MAXSIMULTANEOUSUPLOADS->value() );
	Config->SetMaxUploadRate( floor((SpinBox_MAXUPLOADRATE->cleanText().toFloat()*1024)+0.5) );
	Config->SetMaxDownloadRate( floor((SpinBox_MAXDOWNLOADRATE->cleanText().toFloat()*1024)+0.5) );
	Config->SetDownloadQueueTime(SpinBox_SAVEDOWNLOADQUEUETIME->value());

	Config->SetTCPListenPort(LineEdit_TCPLISTENPORT->text().toInt(&ok));
	Config->SetUDPListenPort(LineEdit_UDPLISTENPORT->text().toInt(&ok));
	Config->SetFakeHost(RadioButton_WRONGIP->isOn());

	if ( RadioButton_ACTIVEMODE->isOn() )
		Config->SetMode(ecmACTIVE);
	else
		Config->SetMode(ecmPASSIVE);

	Config->SetHost(LineEdit_FAKEHOST->text().ascii());
	Config->SetInterface(LineEdit_INTERFACE->text().ascii());
	Config->SetSendMessageOnActiveModeRequest(CheckBox_SENDMESSAGEONACTIVEMODEREQUEST->isChecked());

	// gui settings
	Config->SetOpenPrivateChatWindow(CheckBox_OPENPRIVATECHATWINDOW->isChecked());
	Config->SetEnableEmoticons(CheckBox_EMOTICONS->isChecked());

	if ( RadioButton_UNITBYTE->isChecked() )
		Config->SetUnit(euBYTE);
	else if ( RadioButton_UNITKBYTE->isChecked() )
		Config->SetUnit(euKBYTE);
	else if ( RadioButton_UNITMBYTE->isChecked() )
		Config->SetUnit(euMBYTE);
	else if ( RadioButton_UNITGBYTE->isChecked() )
		Config->SetUnit(euGBYTE);
	else
		Config->SetUnit(euAUTO);
	
	Config->SetTheme(ComboBox_THEME->currentText().ascii());
	Config->SetSendChat(ComboBox_SENDCHAT->currentText().ascii());
	Config->SetLanguageFile(LineEdit_LANGUAGEFILE->text().ascii());
	Config->SetShowStatusMessage(CheckBox_SHOWSTATUSMESSAGE->isChecked());
	Config->SetQueryOnExit(CheckBox_QUERYONEXIT->isChecked());
	Config->SetSendHidePrivateChatToPublicChat(CheckBox_SENDHIDEPRIVATECHATTOPUBLICCHAT->isChecked());
	Config->SetChatSendOfflineMessages(CheckBox_CHATSENDOFFLINEMESSAGES->isChecked());
	Config->SetChatRecvOfflineMessages(CheckBox_CHATRECVOFFLINEMESSAGES->isChecked());
	Config->SetChatMessageMaxLines( SpinBox_CHATMESSAGEMAXLINES->value() );

	// sound settings
	Config->SetSoundDisabled(CheckBox_SOUNDDISABLED->isChecked());
	Config->SetSoundDisabledAway(CheckBox_SOUNDDISABLEDAWAY->isChecked());

	Config->SetSoundEnabled(eusCONNECT,CheckBox_SOUNDPLAYCONNECT->isChecked());
	Config->SetSoundEnabled(eusDISCONNECT,CheckBox_SOUNDPLAYDISCONNECT->isChecked());
	Config->SetSoundEnabled(eusSEND,CheckBox_SOUNDPLAYSEND->isChecked());
	Config->SetSoundEnabled(eusRECEIVE,CheckBox_SOUNDPLAYRECEIVE->isChecked());
	Config->SetSoundEnabled(eusFIRSTRECEIVE,CheckBox_SOUNDPLAYFIRSTRECEIVE->isChecked());

	Config->SetExternalSoundPlayer(CheckBox_EXTERNALPLAYER->isChecked());
	Config->SetExternalPlayer(LineEdit_EXTERNALPLAYER->text().ascii());
	Config->SetSoundFile( eusCONNECT, LineEdit_SOUNDFILECONNECT->text().ascii());
	Config->SetSoundFile( eusDISCONNECT, LineEdit_SOUNDFILEDISCONNECT->text().ascii());
	Config->SetSoundFile( eusSEND, LineEdit_SOUNDFILESEND->text().ascii());
	Config->SetSoundFile( eusRECEIVE, LineEdit_SOUNDFILERECEIVE->text().ascii());
	Config->SetSoundFile( eusFIRSTRECEIVE, LineEdit_SOUNDFILEFIRSTRECEIVE->text().ascii());

	Config->SetAppFont(LineEdit_APPFONT->text().ascii());

	// gui->client
	Config->SetChatShowJoinsAndParts(CheckBox_CHATSHOWJOINSANDPARTS->isChecked());

	// gui->client
	Config->SetOpenClientWindowsMinimized(CheckBox_CLIENTWINDOWSMINIMIZED->isChecked());
	Config->SetShowVersionTag(CheckBox_SHOWVERSIONTAG->isChecked());
	Config->SetUserListRightAlignment(CheckBox_USERLISTRIGHTALIGNMENT->isChecked());
	Config->SetAutoAwayMode(CheckBox_AUTOAWAYMODE->isChecked());
	Config->SetAutoAwayTime(SpinBox_AUTOAWAYTIME->value());

	// log settings
	Config->SetLogFile(CheckBox_LOGFILE->isChecked());
	Config->SetLogFileName( LineEdit_LOGFILENAME->text().ascii() );
	Config->SetTimeStamp(etsPRIVATECHAT,CheckBox_TIMESTAMPPRIVATECHAT->isChecked());
	Config->SetTimeStamp(etsHUBCHAT,CheckBox_TIMESTAMPHUBCHAT->isChecked());

	Config->SetLogChatOption(elcoENABLELOGGING, CheckBox_LOGCHAT_ENABLELOGGING->isChecked() );
	Config->SetLogChatOption(elcoAPPENDDATE, CheckBox_LOGCHAT_APPENDDATE->isChecked() );
	Config->SetLogChatOption(elcoAPPENDHUBNAME, CheckBox_LOGCHAT_APPENDHUBNAME->isChecked() );
	Config->SetLogChatOption(elcoAPPENDHUBHOST, CheckBox_LOGCHAT_APPENDHUBHOST->isChecked() );
	Config->SetLogChatOption(elcoDISABLEPUBLICCHAT, CheckBox_LOGCHAT_DISABLEPUBLICCHAT->isChecked() );
	Config->SetLogChatOption(elcoENABLENICKNAMEFILTER, CheckBox_LOGCHAT_ENABLENICKNAMEFILTER->isChecked() );
	Config->SetLogChatOption(elcoNICKNAMEFILTER_ENABLELOGGING, CheckBox_LOGCHAT_NICKNAMEFILTER_ENABLELOGGING->isChecked() );

	CList<CString> nnflist;
	uint i = 0;

	for(i=0;i<ListBox_LOGCHAT_NICKNAMEFILTER_LIST->count();i++)
	{
		nnflist.Add( new CString(ListBox_LOGCHAT_NICKNAMEFILTER_LIST->text(i).ascii()) );
	}

	Config->SetLogChatNickNameFilter( &nnflist );

	// other settings
	Config->SetHubListStoreLocal(CheckBox_HUBLISTSTORELOCAL->isChecked());
	Config->SetReconnectCount( SpinBox_RECONNECTCOUNT->value() );
	Config->SetReconnectTimeout( SpinBox_RECONNECTTIMEOUT->value() );
	Config->SetTransferResendTimeout(SpinBox_TRANSFERRESENDTIMEOUT->value());
	Config->SetTransferResponseTimeout(SpinBox_TRANSFERRESPONSETIMEOUT->value());
	Config->SetForceMoveEnabled(CheckBox_FORCEMOVEENABLED->isChecked());
	Config->SetReloadHubListTime(SpinBox_RELOADHUBLISTTIME->value());

	// transfer view settings
	Config->SetTransferViewOptions(etvoCHUNKPERCENT,CheckBox_TV_PERCENTCHUNK->isChecked());
	Config->SetTransferViewOptions(etvoFILEPERCENT,CheckBox_TV_PERCENTFILE->isChecked());
	Config->SetTransferViewOptions(etvoCHUNKSIZE,CheckBox_TV_CHUNKSIZE->isChecked());
	Config->SetTransferViewOptions(etvoFILESIZE,CheckBox_TV_FILESIZE->isChecked());
	Config->SetTransferViewOptions(etvoDOWNLOADRATESINGLE,CheckBox_TV_DOWNLOADRATESINGLE->isChecked());
	Config->SetTransferViewOptions(etvoELAPSEDTIMESINGLE,CheckBox_TV_ELAPSEDTIMESINGLE->isChecked());
	Config->SetTransferViewOptions(etvoDOWNLOADRATEMULTI,CheckBox_TV_DOWNLOADRATEMULTI->isChecked());
	Config->SetTransferViewOptions(etvoELAPSEDTIMEMULTI,CheckBox_TV_ELAPSEDTIMEMULTI->isChecked());
	Config->SetUserUploadSlots(SpinBox_USERUPLOADSLOTS->value());

        if ( RadioButton_NONE->isChecked() )
		Config->SetHubOfflineTransferClose( ectNONE );
	else if ( RadioButton_UPLOAD->isChecked() )
		Config->SetHubOfflineTransferClose( ectUPLD );
	else if ( RadioButton_DOWNLOAD->isChecked() )
		Config->SetHubOfflineTransferClose( ectDLD );
	else if ( RadioButton_BOTH->isChecked() )
		Config->SetHubOfflineTransferClose( ectBOTH );

	CList<DCConfigHubListUrl> hublisturllist;

	QCheckListItem * cli = (QCheckListItem*)ListView_HUBLISTURL->firstChild();

	while(cli)
	{
		DCConfigHubListUrl * hublisturl = new DCConfigHubListUrl();

		hublisturl->sUrl     = cli->text(0);
		hublisturl->bEnabled = cli->isOn();

		hublisturllist.Add(hublisturl);

		cli = (QCheckListItem*)cli->nextSibling();
	}

	Config->SetHubListUrlList(&hublisturllist);

	i=0;

	Config->SetChatColor(eccCHATTIMESTAMP,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccCHATLOCALNICK,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccCHATLOCALTEXT,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccCHATREMOTENICK,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccCHATREMOTETEXT,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccCHATSTATUSNICK,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccCHATSTATUSTEXT,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccPUBLICCHATTIMESTAMP,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccPUBLICCHATLOCALNICK,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccPUBLICCHATLOCALTEXT,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccPUBLICCHATREMOTENICK,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccPUBLICCHATREMOTETEXT,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccPUBLICCHATSTATUSNICK,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccPUBLICCHATSTATUSTEXT,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccPUBLICPRIVATECHATNICK,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
	Config->SetChatColor(eccPUBLICPRIVATECHATTEXT,QColor(ListBox_CHATCOLOR->item(i++)->pixmap()->convertToImage().pixel(0,0)).name().ascii());
}

/** */
void DCOptions::SaveSharedFolders()
{
	CList<DCConfigShareFolder> sharedfolders;
	DCConfigShareFolder * csf;

	// save shared folders
	sharedfolders.Clear();

	QListViewItemIterator it( ListView_SHAREDFOLDERS );

	for ( ; it.current(); ++it )
	{
		csf = new DCConfigShareFolder();
		csf->m_sPath  = it.current()->text(0).ascii();
		csf->m_sAlias = it.current()->text(1).ascii();
		sharedfolders.Add(csf);
	}

	Config->SetSharedFolders( &sharedfolders );
}

/** */
void DCOptions::slotCreateShareList()
{
	SaveSharedFolders();

	if ( CFileManager::Instance() )
	{
		CFileManager::Instance()->CreateShareList();
	}
}

/** */
void DCOptions::slotShareListInfo()
{
	QString s;

	if ( CFileManager::Instance()->GetShareSize() == 0 )
	{
		s = tr("Share list not available");
	}
	else
	{
		s = tr("Compressed list size (HE3/BZ): ") +
			QString().setNum(CFileManager::Instance()->GetShareBufferSize(esbtHE3)) + "/" +
			QString().setNum(CFileManager::Instance()->GetShareBufferSize(esbtBZ)) + " " +
			tr("bytes") + "\n" +
			tr("Total shared: ") +
			CUtils::GetSizeString(CFileManager::Instance()->GetShareSize(),Config->GetUnit()).Data();
	}

	QMessageBox::information( this, tr("Share List Info"), s );
}

/** */
void DCOptions::slotSetTheme()
{
	QString s;

	if ( Config->GetThemeSupport() == TRUE )
	{
		s = ComboBox_THEME->currentText();

		if ( s != "" )
		{
			QApplication::setStyle(s);
		}
	}
}

/** */
void DCOptions::slotBrowseLanguageFile()
{
	QString file;

	file = QFileDialog::getOpenFileName( "", "Language File (*.qm)", this, "blf", tr("Select language file") );

	if( file != "" )
	{
		LineEdit_LANGUAGEFILE->setText(file);
	}
}

/** */
void DCOptions::slotSelectLanguage()
{
	int id;
	QPopupMenu *m;
	CString s = "";

	m = new QPopupMenu(this);

	DCMenuHandler::InsertMenu( m, emiLANGUAGE_CZECH );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_DANISH );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_DUTCH );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_FINNISH );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_FRANCE );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_GERMAN );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_HUNGARIAN );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_ICELANDIC );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_ITALY );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_NORWEGIAN );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_POLISH );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_ROMANIAN );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_SPANISH );
	DCMenuHandler::InsertMenu( m, emiLANGUAGE_SWEDISH );

	id = m->exec(QCursor::pos());

	delete m;

	if ( id == emiLANGUAGE_CZECH )
		s = "dcgui.cs.qm";
	else if ( id == emiLANGUAGE_DANISH )
		s = "dcgui.da.qm";
	else if ( id == emiLANGUAGE_DUTCH )
		s = "dcgui.nl.qm";
	else if ( id == emiLANGUAGE_FINNISH )
		s = "dcgui.fi.qm";
	else if ( id == emiLANGUAGE_FRANCE )
		s = "dcgui.fr.qm";
	else if ( id == emiLANGUAGE_GERMAN )
		s = "dcgui.de.qm";
	else if ( id == emiLANGUAGE_HUNGARIAN )
		s = "dcgui.hu.qm";
	else if ( id == emiLANGUAGE_ICELANDIC )
		s = "dcgui.is.qm";
	else if ( id == emiLANGUAGE_ITALY )
		s = "dcgui.it.qm";
	else if ( id == emiLANGUAGE_NORWEGIAN )
		s = "dcgui.nb.qm";
	else if ( id == emiLANGUAGE_POLISH )
		s = "dcgui.pl.qm";
	else if ( id == emiLANGUAGE_ROMANIAN )
		s = "dcgui.ro.qm";
	else if ( id == emiLANGUAGE_SPANISH )
		s = "dcgui.es.qm";
	else if ( id == emiLANGUAGE_SWEDISH )
		s = "dcgui.sv.qm";

	if ( s != "" )
	{
		LineEdit_LANGUAGEFILE->setText(LineEdit_DATAFOLDER->text()+DIRSEPARATOR+"translation"+DIRSEPARATOR+s.Data());
	}
}

void DCOptions::slotBrowseAppFont()
{
  bool ok;
  QFont font = QFontDialog::getFont(&ok, QApplication::font(), this);
  if (ok == TRUE)
  {
	LineEdit_APPFONT->setText(font.toString());
  }
}

/** */
void DCOptions::slotAddHubListUrl()
{
	bool ok = FALSE;
	QString url;

	url = QInputDialog::getText(
		tr("HubList URL"),
		tr("Please enter a url"),
		QLineEdit::Normal, QString::null, &ok, this );

	if (ok)
	{
		new QCheckListItem( ListView_HUBLISTURL, url, QCheckListItem::CheckBox );
	}
}

/** */
void DCOptions::slotRemoveHubListUrl()
{
	QListViewItem * item = ListView_HUBLISTURL->currentItem();

	if (item)
	{
		ListView_HUBLISTURL->takeItem(item);
		delete item;
	}
}

/** */
void DCOptions::slotListViewOptionClicked( QListViewItem * item )
{
	if ( item == plviDCGUI )
	{
		TabWidget_DCGUI->setCurrentPage(0);
	}
	else if ( item == plviDCGUIIdentify )
	{
		TabWidget_DCGUI->setCurrentPage(0);
	}
	else if ( item == plviTransfer )
	{
		TabWidget_DCGUI->setCurrentPage(1);
		TabWidget_TRANSFER->setCurrentPage(0);
	}
	else if ( item == plviTransferFolder )
	{
		TabWidget_DCGUI->setCurrentPage(1);
		TabWidget_TRANSFER->setCurrentPage(0);
	}
	else if ( item == plviTransferSettings )
	{
		TabWidget_DCGUI->setCurrentPage(1);
		TabWidget_TRANSFER->setCurrentPage(1);
	}
	else if ( item == plviConnection )
	{
		TabWidget_DCGUI->setCurrentPage(2);
		TabWidget_CONNECTION->setCurrentPage(0);
	}
	else if ( item == plviConnectionMode )
	{
		TabWidget_DCGUI->setCurrentPage(2);
		TabWidget_CONNECTION->setCurrentPage(0);
	}
	else if ( item == plviConnectionSettings )
	{
		TabWidget_DCGUI->setCurrentPage(2);
		TabWidget_CONNECTION->setCurrentPage(1);
	}
	else if ( item == plviGUI )
	{
		TabWidget_DCGUI->setCurrentPage(3);
		TabWidget_GUI->setCurrentPage(0);
	}
	else if ( item == plviGUIUser )
	{
		TabWidget_DCGUI->setCurrentPage(3);
		TabWidget_GUI->setCurrentPage(3);
	}
	else if ( item == plviGUIChat )
	{
		TabWidget_DCGUI->setCurrentPage(3);
		TabWidget_GUI->setCurrentPage(2);
	}
	else if ( item == plviGUITransfer )
	{
		TabWidget_DCGUI->setCurrentPage(3);
		TabWidget_GUI->setCurrentPage(1);
	}
	else if ( item == plviGUIGeneral )
	{
		TabWidget_DCGUI->setCurrentPage(3);
		TabWidget_GUI->setCurrentPage(0);
	}
	else if ( item == plviSound )
	{
		TabWidget_DCGUI->setCurrentPage(4);
	}
	else if ( item == plviLog )
	{
		TabWidget_DCGUI->setCurrentPage(5);
	}
	else if ( item == plviOther )
	{
		TabWidget_DCGUI->setCurrentPage(6);
	}
	else if ( item == plviSecurity )
	{
		TabWidget_DCGUI->setCurrentPage(7);
	}
}

